from django.contrib.admin.views.decorators import staff_member_required
from django.urls import re_path
from django.views.decorators.cache import never_cache

from . import views


urlpatterns = [
    re_path(r"^upload/", staff_member_required(views.upload), name="ckeditor_upload"),
    re_path(
        r"^browse/",
        never_cache(staff_member_required(views.browse)),
        name="ckeditor_browse",
    ),
]



import os
from io import BytesIO

from django.conf import settings
from django.utils.functional import cached_property
from PIL import Image

from ckeditor_uploader import utils


THUMBNAIL_SIZE = getattr(settings, "CKEDITOR_THUMBNAIL_SIZE", (75, 75))


class PillowBackend:
    def __init__(self, storage_engine, file_object):
        self.file_object = file_object
        self.storage_engine = storage_engine

    @cached_property
    def is_image(self):
        try:
            Image.open(
                BytesIO(self.file_object.read())
            ).verify()  # verify closes the file
            return True
        except OSError:
            return False
        finally:
            self.file_object.seek(0)

    def _compress_image(self, image):
        quality = getattr(settings, "CKEDITOR_IMAGE_QUALITY", 75)
        image = image.resize(image.size, Image.ANTIALIAS).convert("RGB")
        image_tmp = BytesIO()
        image.save(image_tmp, format="JPEG", quality=quality, optimize=True)
        return image_tmp

    def save_as(self, filepath):
        if not self.is_image:
            saved_path = self.storage_engine.save(filepath, self.file_object)
            return saved_path

        image = Image.open(self.file_object)

        should_compress = getattr(settings, "CKEDITOR_FORCE_JPEG_COMPRESSION", False)
        is_animated = hasattr(image, "is_animated") and image.is_animated
        if should_compress and not is_animated:
            file_object = self._compress_image(image)
            filepath = f"{os.path.splitext(filepath)[0]}.jpg"
            saved_path = self.storage_engine.save(filepath, file_object)
        else:
            file_object = self.file_object
            saved_path = self.storage_engine.save(filepath, self.file_object)

        if not is_animated:
            self.create_thumbnail(file_object, saved_path)
        return saved_path

    def create_thumbnail(self, file_object, file_path):
        thumbnail_filename = utils.get_thumb_filename(file_path)
        thumbnail_io = BytesIO()
        # File object after saving e.g. to S3 can be closed.
        try:
            image = Image.open(file_object).convert("RGB")
        except ValueError:
            file_object = self.storage_engine.open(file_path)
            image = Image.open(file_object).convert("RGB")
        image.thumbnail(THUMBNAIL_SIZE, Image.ANTIALIAS)
        image.save(thumbnail_io, format="JPEG", optimize=True)
        return self.storage_engine.save(thumbnail_filename, thumbnail_io)



from django import test

from ckeditor_uploader import backends


class CustomBackend:
    pass


class TestSelectingCustomBackend(test.SimpleTestCase):
    @test.override_settings(
        CKEDITOR_IMAGE_BACKEND="ckeditor_demo.demo_application.tests.test_custom_backend.CustomBackend"
    )
    def test_setting_returns_custom_backend(self):
        result = backends.get_backend()
        self.assertEqual(result, CustomBackend)



import os
import tempfile


"""
Django settings for ckeditor_demo project.

For more information on this file, see
https://docs.djangoproject.com/en/1.6/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.6/ref/settings/
"""


BASE_DIR = os.path.dirname(os.path.dirname(__file__))


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.6/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = "cwog(6mx-+m9-@*n7jsn+*q4in*+nss_nv+s0da39ail@=x(ne"

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
            "debug": DEBUG,
        },
    },
]

ALLOWED_HOSTS = []

CACHED_STORAGE = False

if CACHED_STORAGE:
    DEFAULT_FILE_STORAGE = "django.contrib.staticfiles.storage.CachedStaticFilesStorage"
    STATICFILES_STORAGE = "django.contrib.staticfiles.storage.CachedStaticFilesStorage"
    STATICFILES_FINDERS = (
        "django.contrib.staticfiles.finders.FileSystemFinder",
        "django.contrib.staticfiles.finders.AppDirectoriesFinder",
        "django.contrib.staticfiles.finders.DefaultStorageFinder",
    )

# Application definition

INSTALLED_APPS = (
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "django_extensions",
    "ckeditor",
    "ckeditor_uploader",
    "ckeditor_demo.demo_application",
)

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

ROOT_URLCONF = "ckeditor_demo.urls"

WSGI_APPLICATION = "ckeditor_demo.wsgi.application"


# Database
# https://docs.djangoproject.com/en/1.6/ref/settings/#databases

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": os.path.join(BASE_DIR, "db.sqlite3"),
    }
}

DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"

# Internationalization
# https://docs.djangoproject.com/en/1.6/topics/i18n/

LANGUAGE_CODE = "en-us"

TIME_ZONE = "UTC"

USE_I18N = True

USE_L10N = True

USE_TZ = True

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.6/howto/static-files/

STATIC_URL = "/static/"
MEDIA_URL = "/media/"
STATIC_ROOT = os.path.join(tempfile.gettempdir(), "ck_static")
MEDIA_ROOT = os.path.join(tempfile.gettempdir(), "ck_media")

from ckeditor.configs import DEFAULT_CONFIG  # noqa


CKEDITOR_UPLOAD_PATH = "uploads/"
CKEDITOR_IMAGE_BACKEND = "ckeditor_uploader.backends.PillowBackend"
CKEDITOR_THUMBNAIL_SIZE = (300, 300)
CKEDITOR_IMAGE_QUALITY = 40
CKEDITOR_BROWSE_SHOW_DIRS = True
CKEDITOR_ALLOW_NONIMAGE_FILES = True

CUSTOM_TOOLBAR = [
    {
        "name": "document",
        "items": [
            "Styles",
            "Format",
            "Bold",
            "Italic",
            "Underline",
            "Strike",
            "-",
            "TextColor",
            "BGColor",
            "-",
            "JustifyLeft",
            "JustifyCenter",
            "JustifyRight",
            "JustifyBlock",
        ],
    },
    {
        "name": "widgets",
        "items": [
            "Undo",
            "Redo",
            "-",
            "NumberedList",
            "BulletedList",
            "-",
            "Outdent",
            "Indent",
            "-",
            "Link",
            "Unlink",
            "-",
            "Image",
            "CodeSnippet",
            "Table",
            "HorizontalRule",
            "Smiley",
            "SpecialChar",
            "-",
            "Blockquote",
            "-",
            "ShowBlocks",
            "Maximize",
        ],
    },
]

CKEDITOR_CONFIGS = {
    "default": DEFAULT_CONFIG,
    "my-custom-toolbar": {
        "skin": "moono-lisa",
        "toolbar": CUSTOM_TOOLBAR,
        "toolbarGroups": None,
        "extraPlugins": ",".join(["image2", "codesnippet"]),
        "removePlugins": ",".join(["image"]),
        "codeSnippet_theme": "xcode",
    },
}



import warnings

from django.conf import settings
from django.utils.module_loading import import_string

from .dummy_backend import DummyBackend


__all__ = ["get_backend", "DummyBackend", "PillowBackend"]


try:
    from .pillow_backend import PillowBackend
except ImportError:
    pass


# Allow for a custom image backend defined in settings.
def get_backend():
    backend_path = getattr(
        settings, "CKEDITOR_IMAGE_BACKEND", "ckeditor_uploader.backends.DummyBackend"
    )

    # Honour old registry keys while emitting warnings
    if backend_path is None:
        backend_path = "ckeditor_uploader.backends.DummyBackend"
        warnings.warn(
            "CKEDITOR_IMAGE_BACKEND now uses a fully qualified path to the backend class."
            "  None should be changed to 'ckeditor_uploader.backends.DummyBackend'",
            PendingDeprecationWarning,
            stacklevel=2,
        )
    elif backend_path == "pillow":
        backend_path = "ckeditor_uploader.backends.PillowBackend"
        warnings.warn(
            "CKEDITOR_IMAGE_BACKEND now uses a fully qualified path to the backend class."
            "  'pillow' should be changed to 'ckeditor_uploader.backends.PillowBackend'",
            PendingDeprecationWarning,
            stacklevel=2,
        )

    return import_string(backend_path)



from django.conf import settings
from django.conf.urls.static import static
from django.contrib import admin
from django.urls import include, path, re_path

from .demo_application import views


urlpatterns = (
    [
        path("", views.ckeditor_form_view, name="ckeditor-form"),
        path(
            "multiwidget/",
            views.ckeditor_multi_widget_form_view,
            name="ckeditor-multi-widget-form",
        ),
        re_path(r"^admin/", admin.site.urls),
        re_path(r"^ckeditor/", include("ckeditor_uploader.urls")),
    ]
    + static(settings.STATIC_URL, document_root=settings.STATIC_ROOT)
    + static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)
)



from ckeditor import fields

from . import widgets


class RichTextUploadingField(fields.RichTextField):
    def formfield(self, **kwargs):
        defaults = {
            "form_class": RichTextUploadingFormField,
        }
        defaults.update(kwargs)
        return super().formfield(**defaults)


class RichTextUploadingFormField(fields.RichTextFormField):
    widget = widgets.CKEditorUploadingWidget



from django import forms
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.core.serializers.json import DjangoJSONEncoder
from django.forms.widgets import Media
from django.templatetags.static import static
from django.utils.encoding import force_str
from django.utils.functional import Promise
from django.utils.translation import get_language
from js_asset import JS

from .configs import DEFAULT_CONFIG


class LazyEncoder(DjangoJSONEncoder):
    def default(self, obj):
        if isinstance(obj, Promise):
            return force_str(obj)
        return super().default(obj)


json_encode = LazyEncoder().encode


class CKEditorWidget(forms.Textarea):
    """
    Widget providing CKEditor for Rich Text Editing.
    Supports direct image uploads and embed.
    """

    template_name = "ckeditor/widget.html"

    def __init__(
        self,
        config_name="default",
        extra_plugins=None,
        external_plugin_resources=None,
        *args,
        **kwargs
    ):
        super().__init__(*args, **kwargs)

        self.config_name = config_name
        # Setup config from defaults.
        self.config = DEFAULT_CONFIG.copy()

        # Try to get valid config from settings.
        configs = getattr(settings, "CKEDITOR_CONFIGS", None)
        if configs:
            if isinstance(configs, dict):
                # Make sure the config_name exists.
                if self.config_name in configs:
                    config = configs[self.config_name]
                    # Make sure the configuration is a dictionary.
                    if not isinstance(config, dict):
                        raise ImproperlyConfigured(
                            'CKEDITOR_CONFIGS["%s"] \
                                setting must be a dictionary type.'
                            % self.config_name
                        )
                    # Override defaults with settings config.
                    self.config.update(config)
                else:
                    raise ImproperlyConfigured(
                        "No configuration named '%s' \
                            found in your CKEDITOR_CONFIGS setting."
                        % self.config_name
                    )
            else:
                raise ImproperlyConfigured(
                    "CKEDITOR_CONFIGS setting must be a\
                        dictionary type."
                )

        extra_plugins = extra_plugins or self.config.pop("extra_plugins", None) or []

        if extra_plugins:
            self.config["extraPlugins"] = ",".join(extra_plugins)

        self.external_plugin_resources = (
            external_plugin_resources
            or self.config.pop("external_plugin_resources", None)
            or []
        )

    @property
    def media(self):
        return Media(
            js=(
                JS(
                    "ckeditor/ckeditor-init.js",
                    {
                        "id": "ckeditor-init-script",
                        "data-ckeditor-basepath": getattr(
                            settings,
                            "CKEDITOR_BASEPATH",
                            None,
                        )
                        or static("ckeditor/ckeditor/"),
                    },
                ),
                "ckeditor/ckeditor/ckeditor.js",
            )
        )

    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)

        self._set_config()
        context["widget"]["config"] = json_encode(self.config)

        external_plugin_resources = [
            [force_str(a), force_str(b), force_str(c)]
            for a, b, c in self.external_plugin_resources
        ]
        context["widget"]["external_plugin_resources"] = json_encode(
            external_plugin_resources
        )
        return context

    def _set_config(self):
        lang = get_language().lower()
        if lang == "zh-hans":
            lang = "zh-cn"
        elif lang == "zh-hant":
            lang = "zh"
        self.config["language"] = lang



from http import HTTPStatus

from django.contrib.auth import get_user_model
from django.test import TestCase
from django.urls import reverse

from ckeditor.fields import RichTextFormField
from ckeditor.widgets import CKEditorWidget
from ckeditor_uploader.fields import RichTextUploadingFormField
from ckeditor_uploader.widgets import CKEditorUploadingWidget

from ..forms import (
    ExampleModelForm,
    ExampleModelOverriddenWidgetForm,
    ExampleNonUploadModelForm,
)


class ModelFieldAndFormFieldTestCase(TestCase):
    def test_non_upload_model_form_contains_expected_formfield(self):
        form = ExampleNonUploadModelForm()
        self.assert_form_contains_expected_formfield_with_default_widget_options(
            form, "content", RichTextFormField, CKEditorWidget
        )

    def test_upload_model_form_contains_expected_formfield(self):
        form = ExampleModelForm()
        self.assert_form_contains_expected_formfield_with_default_widget_options(
            form, "content", RichTextUploadingFormField, CKEditorUploadingWidget
        )

    def assert_form_contains_expected_formfield_with_default_widget_options(
        self, form, field_name, expected_field_type, expected_widget_type
    ):
        form_field = form.fields[field_name]
        self.assertIs(type(form_field), expected_field_type)
        widget = form_field.widget
        self.assertIs(type(widget), expected_widget_type)
        self.assertEqual(widget.config_name, "default")
        self.assertEqual(widget.config.get("extraPlugins"), None)
        self.assertListEqual(widget.external_plugin_resources, [])

    def test_upload_model_form_with_overridden_widget_contains_expected_formfield(self):
        form = ExampleModelOverriddenWidgetForm()
        form_field = form.fields["content"]
        self.assertIs(type(form_field), RichTextUploadingFormField)
        widget = form_field.widget
        self.assertIs(type(widget), CKEditorUploadingWidget)
        self.assertEqual(widget.config_name, "my-custom-toolbar")
        self.assertEqual(widget.config.get("extraPlugins"), "someplugin,anotherplugin")
        self.assertListEqual(
            widget.external_plugin_resources,
            [
                (
                    "someplugin",
                    "/static/path/to/someplugin/",
                    "plugin.js",
                )
            ],
        )

    def test_admin_model_forms_contain_expected_formfields(self):
        superuser = get_user_model().objects.create_user(
            username="superuser", is_superuser=True, is_staff=True
        )
        self.client.force_login(superuser)

        response = self.client.get(
            reverse("admin:demo_application_examplenonuploadmodel_add")
        )
        self.assertEqual(response.status_code, HTTPStatus.OK)

        form = response.context["adminform"].form
        self.assert_form_contains_expected_formfield_with_default_widget_options(
            form, "content", RichTextFormField, CKEditorWidget
        )

        response = self.client.get(reverse("admin:demo_application_examplemodel_add"))
        self.assertEqual(response.status_code, HTTPStatus.OK)

        form = response.context["adminform"].form
        self.assert_form_contains_expected_formfield_with_default_widget_options(
            form, "content", RichTextUploadingFormField, CKEditorUploadingWidget
        )



from django.urls import reverse

from ckeditor import widgets


class CKEditorUploadingWidget(widgets.CKEditorWidget):
    def _set_config(self):
        if "filebrowserUploadUrl" not in self.config:
            self.config.setdefault("filebrowserUploadUrl", reverse("ckeditor_upload"))
        if "filebrowserBrowseUrl" not in self.config:
            self.config.setdefault("filebrowserBrowseUrl", reverse("ckeditor_browse"))
        super()._set_config()



DEFAULT_CONFIG = {
    "skin": "moono-lisa",
    "toolbar_Basic": [["Source", "-", "Bold", "Italic"]],
    "toolbar_Full": [
        [
            "Styles",
            "Format",
            "Bold",
            "Italic",
            "Underline",
            "Strike",
            "SpellChecker",
            "Undo",
            "Redo",
        ],
        ["Link", "Unlink", "Anchor"],
        ["Image", "Flash", "Table", "HorizontalRule"],
        ["TextColor", "BGColor"],
        ["Smiley", "SpecialChar"],
        ["Source"],
    ],
    "toolbar": "Full",
    "height": 291,
    "width": 835,
    "filebrowserWindowWidth": 940,
    "filebrowserWindowHeight": 725,
}



import os
import warnings

from django.contrib.auth.models import User
from django.test import RequestFactory, SimpleTestCase, TestCase
from django.test.utils import override_settings

from ckeditor_uploader import backends
from ckeditor_uploader.views import get_upload_filename

from .utils import get_absolute_name


MOCK_FILENAME = "some_filename.jpg"
GENERATOR_PREFIX = "some_folder"
GENERATED_MOCK_FILENAME = os.path.join(GENERATOR_PREFIX, MOCK_FILENAME)


def generator_with_filename_and_request(filename, _request):
    return os.path.join(GENERATOR_PREFIX, filename)


def generator_with_only_filename(filename):
    return os.path.join(GENERATOR_PREFIX, filename)


def generator_without_parameters():
    return GENERATOR_PREFIX


class TestUploadFilenameGeneratorParameters(TestCase):
    fixtures = ["test_admin.json"]

    def setUp(self):
        self.mock_request = RequestFactory().request()
        self.mock_request.user = User.objects.get(username="test")

    @override_settings(
        CKEDITOR_FILENAME_GENERATOR=get_absolute_name(
            generator_with_filename_and_request
        )
    )
    def test_compatible_parameters(self):
        # `self.assertWarns()` is not available in Python 2
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("always")

            generated_filename = get_upload_filename(MOCK_FILENAME, self.mock_request)
            self.assertEqual(len(w), 0)

        self.assertIn(GENERATED_MOCK_FILENAME, generated_filename)

    @override_settings(
        CKEDITOR_FILENAME_GENERATOR=get_absolute_name(generator_with_only_filename)
    )
    def test_no_request_parameter_deprecation_warning(self):
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("always")

            generated_filename = get_upload_filename(MOCK_FILENAME, self.mock_request)
            self.assertEqual(len(w), 1)
            self.assertTrue(issubclass(w[0].category, PendingDeprecationWarning))

        self.assertIn(GENERATED_MOCK_FILENAME, generated_filename)

    @override_settings(
        CKEDITOR_FILENAME_GENERATOR=get_absolute_name(generator_without_parameters)
    )
    def test_incompatible_parameters(self):
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("always")

            generated_filename = get_upload_filename(MOCK_FILENAME, self.mock_request)
            self.assertEqual(len(w), 1)

        self.assertIn(MOCK_FILENAME, generated_filename)
        self.assertNotIn(GENERATOR_PREFIX, generated_filename)


class TestDeprecatedImageBackendValues(SimpleTestCase):
    @override_settings(CKEDITOR_IMAGE_BACKEND="pillow")
    def test_pillow_setting(self):
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("always")
            result = backends.get_backend()
            self.assertEqual(len(w), 1)
            self.assertEqual(result, backends.PillowBackend)

    @override_settings(CKEDITOR_IMAGE_BACKEND=None)
    def test_none_setting(self):
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("always")
            result = backends.get_backend()
            self.assertEqual(len(w), 1)
            self.assertEqual(result, backends.DummyBackend)



import os.path
import random
import re
import string

from django.conf import settings
from django.template.defaultfilters import slugify
from django.utils.encoding import force_str
from django.utils.module_loading import import_string


# Non-image file icons, matched from top to bottom

fileicons_path = "{}/file-icons/".format(
    getattr(settings, "CKEDITOR_FILEICONS_PATH", "/static/ckeditor")
)
# This allows adding or overriding the default icons used by Gallerific by getting an additional two-tuple list from
# the project settings.  If it does not exist, it is ignored.  If the same file extension exists twice, the settings
# file version is used instead of the default.
override_icons = getattr(settings, "CKEDITOR_FILEICONS", [])
ckeditor_icons = [
    (r"\.pdf$", fileicons_path + "pdf.png"),
    (r"\.doc$|\.docx$|\.odt$", fileicons_path + "doc.png"),
    (r"\.txt$", fileicons_path + "txt.png"),
    (r"\.ppt$", fileicons_path + "ppt.png"),
    (r"\.xls$", fileicons_path + "xls.png"),
    (".*", fileicons_path + "file.png"),  # Default
]
CKEDITOR_FILEICONS = override_icons + ckeditor_icons

IMAGE_EXTENSIONS = {".jpg", ".jpeg", ".png", ".gif"}


# Allow for a custom storage backend defined in settings.
def get_storage_class():
    return import_string(
        getattr(
            settings,
            "CKEDITOR_STORAGE_BACKEND",
            "django.core.files.storage.DefaultStorage",
        )
    )()


storage = get_storage_class()


def slugify_filename(filename):
    """Slugify filename"""
    name, ext = os.path.splitext(filename)
    slugified = get_slugified_name(name)
    return slugified + ext


def get_slugified_name(filename):
    slugified = slugify(filename)
    return slugified or get_random_string()


def get_random_string():
    return "".join(random.sample(string.ascii_lowercase * 6, 6))


def get_icon_filename(file_name):
    """
    Return the path to a file icon that matches the file name.
    """
    for regex, iconpath in CKEDITOR_FILEICONS:
        if re.search(regex, file_name, re.I):
            return iconpath


def get_thumb_filename(file_name):
    """
    Generate thumb filename by adding _thumb to end of
    filename before . (if present)
    """
    return force_str("{0}_thumb{1}").format(*os.path.splitext(file_name))


def get_media_url(path):
    """
    Determine system file's media URL.
    """
    return storage.url(path)


def is_valid_image_extension(file_path):
    extension = os.path.splitext(file_path.lower())[1]
    return extension in IMAGE_EXTENSIONS



from django.db import models

from ckeditor.fields import RichTextField
from ckeditor_uploader.fields import RichTextUploadingField


class ExampleModel(models.Model):
    content = RichTextUploadingField()


class ExampleNonUploadModel(models.Model):
    content = RichTextField()



from django import forms
from django.db import models

from .widgets import CKEditorWidget


class RichTextField(models.TextField):
    def __init__(self, *args, **kwargs):
        self.config_name = kwargs.pop("config_name", "default")
        self.extra_plugins = kwargs.pop("extra_plugins", [])
        self.external_plugin_resources = kwargs.pop("external_plugin_resources", [])
        super().__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        defaults = {
            "form_class": RichTextFormField,
            "config_name": self.config_name,
            "extra_plugins": self.extra_plugins,
            "external_plugin_resources": self.external_plugin_resources,
        }
        defaults.update(kwargs)
        return super().formfield(**defaults)


class RichTextFormField(forms.CharField):
    widget = CKEditorWidget

    def __init__(
        self,
        config_name="default",
        extra_plugins=None,
        external_plugin_resources=None,
        *args,
        **kwargs
    ):
        kwargs["widget"] = self.widget(
            config_name=config_name,
            extra_plugins=extra_plugins,
            external_plugin_resources=external_plugin_resources,
        )
        super().__init__(*args, **kwargs)



import hashlib
import os.path
import shutil
from datetime import datetime
from typing import Any, Dict

from django.conf import settings

from ckeditor.configs import DEFAULT_CONFIG


def get_upload_directory():
    date_path = datetime.now().strftime("%Y/%m/%d")

    # Complete upload path (upload_path + date_path).
    upload_path = os.path.join(settings.CKEDITOR_UPLOAD_PATH, date_path)
    return os.path.join(settings.MEDIA_ROOT, upload_path)


def get_media_url(fname):
    args = [settings.CKEDITOR_UPLOAD_PATH, datetime.now().strftime("%Y/%m/%d"), fname]
    return settings.MEDIA_URL + "/".join(arg.strip("/") for arg in args)


def remove_upload_directory():
    # Called on test setup
    # Avoid falling in the use case chere django append a hash to the file name
    # to prevent file collisions
    shutil.rmtree(get_upload_directory(), ignore_errors=True)


def sha1(path):
    image = open(path, "rb")
    fhash = hashlib.sha1()
    fhash.update(image.read())
    return fhash.hexdigest()


def get_absolute_media_path(fname):
    upload_directory = get_upload_directory()
    return os.path.join(upload_directory, fname)


def get_absolute_name(class_or_function):
    return f"{class_or_function.__module__}.{class_or_function.__name__}"


def get_config(config_name):
    return {
        **DEFAULT_CONFIG,
        **settings.CKEDITOR_CONFIGS[config_name],
    }


def get_contexts_for_widgets(response) -> Dict[str, Dict[str, Any]]:
    """
    Searches through the response's context for subcontexts of widgets,
    and returns a dictionary with the widgets' `id` attribute as key
    and the widget's context dictionary as value.
    """
    widget_contexts = {}
    for subcontext in response.context:
        if "widget" not in subcontext:
            continue

        for subcontext_dict in subcontext:
            if type(subcontext_dict) is not dict or "widget" not in subcontext_dict:
                continue

            widget = subcontext_dict["widget"]
            widget_id = widget["attrs"]["id"]
            widget_contexts[widget_id] = subcontext_dict

    return widget_contexts



from django import forms

from ckeditor.fields import RichTextFormField
from ckeditor.widgets import CKEditorWidget
from ckeditor_uploader.fields import RichTextUploadingFormField
from ckeditor_uploader.widgets import CKEditorUploadingWidget

from .models import ExampleModel, ExampleNonUploadModel
from .widgets import CkEditorMultiWidget


class CkEditorForm(forms.Form):
    ckeditor_standard_example = RichTextFormField()
    ckeditor_upload_example = RichTextUploadingFormField(
        config_name="my-custom-toolbar"
    )


class CkEditorMultiWidgetForm(forms.Form):
    SUBWIDGET_SUFFIXES = ["0", "1"]

    ckeditor_standard_multi_widget_example = forms.CharField(
        widget=CkEditorMultiWidget(
            widgets={suffix: CKEditorWidget for suffix in SUBWIDGET_SUFFIXES},
        ),
    )
    ckeditor_upload_multi_widget_example = forms.CharField(
        widget=CkEditorMultiWidget(
            widgets={
                suffix: CKEditorUploadingWidget(config_name="my-custom-toolbar")
                for suffix in SUBWIDGET_SUFFIXES
            },
        ),
    )


class ExampleModelForm(forms.ModelForm):
    class Meta:
        model = ExampleModel
        fields = "__all__"


class ExampleNonUploadModelForm(forms.ModelForm):
    class Meta:
        model = ExampleNonUploadModel
        fields = "__all__"


class ExampleModelOverriddenWidgetForm(forms.ModelForm):
    class Meta:
        model = ExampleModel
        fields = "__all__"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.fields["content"].widget = CKEditorUploadingWidget(
            config_name="my-custom-toolbar",
            extra_plugins=["someplugin", "anotherplugin"],
            external_plugin_resources=[
                (
                    "someplugin",
                    "/static/path/to/someplugin/",
                    "plugin.js",
                )
            ],
        )



from http import HTTPStatus

from django.test import TestCase
from django.urls import reverse

from ckeditor.widgets import json_encode

from ..forms import CkEditorMultiWidgetForm
from .utils import get_config, get_contexts_for_widgets


class WidgetContextTestCase(TestCase):
    def setUp(self):
        # Facilitates printing the whole diff if the tests fail
        self.maxDiff = None

    def test_rendered_ckeditor_widgets_contain_expected_context(self):
        response = self.client.get(reverse("ckeditor-form"))
        self.assertEqual(response.status_code, HTTPStatus.OK)

        widget_contexts = get_contexts_for_widgets(response)

        field_name__config__tuples = (
            (
                "ckeditor_standard_example",
                {
                    **get_config("default"),
                    "language": "en-us",
                },
            ),
            (
                "ckeditor_upload_example",
                {
                    **get_config("my-custom-toolbar"),
                    "filebrowserUploadUrl": reverse("ckeditor_upload"),
                    "filebrowserBrowseUrl": reverse("ckeditor_browse"),
                    "language": "en-us",
                },
            ),
        )
        self.assertEqual(len(widget_contexts), len(field_name__config__tuples))

        for field_name, expected_config in field_name__config__tuples:
            with self.subTest(field_name=field_name):
                widget_id = f"id_{field_name}"
                self.assertIn(widget_id, widget_contexts)
                widget_context = widget_contexts[widget_id]

                expected_widget_context = self.get_expected_widget_context(
                    field_name,
                    widget_id,
                    expected_config,
                )
                self.assertDictEqual(widget_context, expected_widget_context)

    @staticmethod
    def get_expected_widget_context(field_name, widget_id, expected_config: dict):
        return {
            "widget": {
                "name": field_name,
                "is_hidden": False,
                "required": True,
                "value": None,
                "attrs": {
                    "cols": "40",
                    "rows": "10",
                    "required": True,
                    "id": widget_id,
                },
                "template_name": "ckeditor/widget.html",
                "config": json_encode(expected_config),
                "external_plugin_resources": "[]",
            },
        }

    def test_rendered_ckeditor_multi_widgets_contain_expected_context(self):
        response = self.client.get(reverse("ckeditor-multi-widget-form"))
        self.assertEqual(response.status_code, HTTPStatus.OK)

        widget_contexts = get_contexts_for_widgets(response)

        field_name__config__tuples = (
            (
                "ckeditor_standard_multi_widget_example",
                {
                    **get_config("default"),
                    "language": "en-us",
                },
            ),
            (
                "ckeditor_upload_multi_widget_example",
                {
                    **get_config("my-custom-toolbar"),
                    "filebrowserUploadUrl": reverse("ckeditor_upload"),
                    "filebrowserBrowseUrl": reverse("ckeditor_browse"),
                    "language": "en-us",
                },
            ),
        )
        self.assertEqual(len(widget_contexts), len(field_name__config__tuples))

        for field_name, expected_config in field_name__config__tuples:
            with self.subTest(field_name=field_name):
                widget_id = f"id_{field_name}"
                self.assertIn(widget_id, widget_contexts)
                widget_context = widget_contexts[widget_id]

                expected_widget_context = self.get_expected_multi_widget_context(
                    field_name, widget_id, expected_config
                )
                self.assertDictEqual(widget_context, expected_widget_context)

    @classmethod
    def get_expected_multi_widget_context(
        cls, field_name, widget_id, expected_config: dict
    ):
        subwidget_contexts = []
        for subwidget_suffix in CkEditorMultiWidgetForm.SUBWIDGET_SUFFIXES:
            expected_widget_context = cls.get_expected_widget_context(
                f"{field_name}_{subwidget_suffix}",
                f"{widget_id}_{subwidget_suffix}",
                expected_config,
            )
            # The multi-widget only uses the contents of the `widget` context variable
            subwidget_context = expected_widget_context["widget"]
            # Each subwidget is non-required (even if its `required` attr is True)
            subwidget_context["required"] = False
            subwidget_contexts.append(subwidget_context)
        return {
            "widget": {
                "name": field_name,
                "is_hidden": False,
                "required": True,
                "value": None,
                "attrs": {
                    "required": True,
                    "id": widget_id,
                },
                "template_name": "django/forms/widgets/multiwidget.html",
                "subwidgets": subwidget_contexts,
            },
        }



import os

from django.conf import settings
from django.core.management.base import BaseCommand

from ckeditor_uploader.backends import get_backend
from ckeditor_uploader.utils import get_thumb_filename
from ckeditor_uploader.views import get_image_files


class Command(BaseCommand):
    """
    Creates thumbnail files for the CKEditor file image browser.
    Useful if starting to use django-ckeditor with existing images.
    """

    def handle(self, *args, **options):
        if getattr(settings, "CKEDITOR_IMAGE_BACKEND", None):
            backend = get_backend()
            for image in get_image_files():
                if not self._thumbnail_exists(image):
                    self.stdout.write("Creating thumbnail for %s" % image)
                    try:
                        backend.create_thumbnail(image)
                    except Exception as e:
                        self.stdout.write(f"Couldn't create thumbnail for {image}: {e}")
            self.stdout.write("Finished")
        else:
            self.stdout.write("No thumbnail backend is enabled")

    def _thumbnail_exists(self, image_path):
        thumb_path = self._to_absolute_path(get_thumb_filename(image_path))
        return os.path.isfile(thumb_path)

    @staticmethod
    def _to_absolute_path(image_path):
        return os.path.join(settings.MEDIA_ROOT, image_path)



import inspect
import os
import warnings
from datetime import datetime

from django.conf import settings
from django.http import HttpResponse, JsonResponse
from django.shortcuts import render
from django.utils.html import escape
from django.utils.module_loading import import_string
from django.views import generic
from django.views.decorators.csrf import csrf_exempt

from ckeditor_uploader import utils
from ckeditor_uploader.backends import get_backend
from ckeditor_uploader.forms import SearchForm
from ckeditor_uploader.utils import storage

from .utils import is_valid_image_extension


def _get_user_path(user):
    user_path = ""

    # If CKEDITOR_RESTRICT_BY_USER is True upload file to user specific path.
    RESTRICT_BY_USER = getattr(settings, "CKEDITOR_RESTRICT_BY_USER", False)
    if RESTRICT_BY_USER:
        try:
            user_prop = getattr(user, RESTRICT_BY_USER)
        except (AttributeError, TypeError):
            user_prop = getattr(user, "get_username")

        if callable(user_prop):
            user_path = user_prop()
        else:
            user_path = user_prop

    return str(user_path)


def get_upload_filename(upload_name, request):
    user_path = _get_user_path(request.user)

    # Generate date based path to put uploaded file.
    # If CKEDITOR_RESTRICT_BY_DATE is True upload file to date specific path.
    if getattr(settings, "CKEDITOR_RESTRICT_BY_DATE", True):
        date_path = datetime.now().strftime("%Y/%m/%d")
    else:
        date_path = ""

    # Complete upload path (upload_path + date_path).
    upload_path = os.path.join(settings.CKEDITOR_UPLOAD_PATH, user_path, date_path)

    if getattr(settings, "CKEDITOR_UPLOAD_SLUGIFY_FILENAME", True) and not hasattr(
        settings, "CKEDITOR_FILENAME_GENERATOR"
    ):
        upload_name = utils.slugify_filename(upload_name)

    if hasattr(settings, "CKEDITOR_FILENAME_GENERATOR"):
        generator = import_string(settings.CKEDITOR_FILENAME_GENERATOR)
        # Does the generator accept a request argument?
        try:
            inspect.signature(generator).bind(upload_name, request)
        except TypeError:
            # Does the generator accept only an upload_name argument?
            try:
                inspect.signature(generator).bind(upload_name)
            except TypeError:
                warnings.warn(
                    "Update %s() to accept the arguments `filename, request`."
                    % settings.CKEDITOR_FILENAME_GENERATOR
                )
            else:
                warnings.warn(
                    "Update %s() to accept a second `request` argument."
                    % settings.CKEDITOR_FILENAME_GENERATOR,
                    PendingDeprecationWarning,
                )
                upload_name = generator(upload_name)
        else:
            upload_name = generator(upload_name, request)

    return storage.get_available_name(os.path.join(upload_path, upload_name))


class ImageUploadView(generic.View):
    http_method_names = ["post"]

    def post(self, request, **kwargs):
        """
        Uploads a file and send back its URL to CKEditor.
        """
        uploaded_file = request.FILES["upload"]

        backend = get_backend()

        ck_func_num = request.GET.get("CKEditorFuncNum")
        if ck_func_num:
            ck_func_num = escape(ck_func_num)

        filewrapper = backend(storage, uploaded_file)
        allow_nonimages = getattr(settings, "CKEDITOR_ALLOW_NONIMAGE_FILES", True)
        # Throws an error when an non-image file are uploaded.
        if not filewrapper.is_image and not allow_nonimages:
            return HttpResponse(
                """
                <script type='text/javascript'>
                window.parent.CKEDITOR.tools.callFunction({}, '', 'Invalid file type.');
                </script>""".format(
                    ck_func_num
                )
            )

        filepath = get_upload_filename(uploaded_file.name, request)

        saved_path = filewrapper.save_as(filepath)

        url = utils.get_media_url(saved_path)

        if ck_func_num:
            # Respond with Javascript sending ckeditor upload url.
            return HttpResponse(
                """
            <script type='text/javascript'>
                window.parent.CKEDITOR.tools.callFunction({}, '{}');
            </script>""".format(
                    ck_func_num, url
                )
            )
        else:
            _, filename = os.path.split(saved_path)
            retdata = {"url": url, "uploaded": "1", "fileName": filename}
            return JsonResponse(retdata)


upload = csrf_exempt(ImageUploadView.as_view())


def get_image_files(user=None, path=""):
    """
    Recursively walks all dirs under upload dir and generates a list of
    full paths for each file found.
    """
    # If a user is provided and CKEDITOR_RESTRICT_BY_USER is True,
    # limit images to user specific path, but not for superusers.
    STORAGE_DIRECTORIES = 0
    STORAGE_FILES = 1

    # allow browsing from anywhere if user is superuser
    # otherwise use the user path
    if user and not user.is_superuser:
        user_path = _get_user_path(user)
    else:
        user_path = ""

    browse_path = os.path.join(settings.CKEDITOR_UPLOAD_PATH, user_path, path)

    try:
        storage_list = storage.listdir(browse_path)
    except NotImplementedError:
        return
    except OSError:
        return

    for filename in storage_list[STORAGE_FILES]:
        if os.path.splitext(filename)[0].endswith("_thumb") or os.path.basename(
            filename
        ).startswith("."):
            continue
        filename = os.path.join(browse_path, filename)
        yield filename

    for directory in storage_list[STORAGE_DIRECTORIES]:
        if directory.startswith("."):
            continue
        directory_path = os.path.join(path, directory)
        yield from get_image_files(user=user, path=directory_path)


def get_files_browse_urls(user=None):
    """
    Recursively walks all dirs under upload dir and generates a list of
    thumbnail and full image URL's for each file found.
    """
    files = []
    for filename in get_image_files(user=user):
        src = utils.get_media_url(filename)
        if getattr(settings, "CKEDITOR_IMAGE_BACKEND", None):
            if is_valid_image_extension(src):
                thumb = utils.get_media_url(utils.get_thumb_filename(filename))
            else:
                thumb = utils.get_icon_filename(filename)
            visible_filename = os.path.split(filename)[1]
            if len(visible_filename) > 20:
                visible_filename = visible_filename[0:19] + "..."
        else:
            thumb = src
            visible_filename = os.path.split(filename)[1]
        files.append(
            {
                "thumb": thumb,
                "src": src,
                "is_image": is_valid_image_extension(src),
                "visible_filename": visible_filename,
            }
        )

    return files


def browse(request):
    files = get_files_browse_urls(request.user)
    if request.method == "POST":
        form = SearchForm(request.POST)
        if form.is_valid():
            query = form.cleaned_data.get("q", "").lower()
            files = list(
                filter(lambda d: query in d["visible_filename"].lower(), files)
            )
    else:
        form = SearchForm()

    show_dirs = getattr(settings, "CKEDITOR_BROWSE_SHOW_DIRS", False)
    dir_list = sorted({os.path.dirname(f["src"]) for f in files}, reverse=True)

    # Ensures there are no objects created from Thumbs.db files - ran across
    # this problem while developing on Windows
    if os.name == "nt":
        files = [f for f in files if os.path.basename(f["src"]) != "Thumbs.db"]

    context = {"show_dirs": show_dirs, "dirs": dir_list, "files": files, "form": form}
    return render(request, "ckeditor/browse.html", context)



# Generated by Django 4.0.3 on 2022-04-10 15:00

from django.db import migrations, models

import ckeditor.fields
import ckeditor_uploader.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="ExampleModel",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("content", ckeditor_uploader.fields.RichTextUploadingField()),
            ],
        ),
        migrations.CreateModel(
            name="ExampleNonUploadModel",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("content", ckeditor.fields.RichTextField()),
            ],
        ),
    ]



#!/usr/bin/env python3

import os
import sys

from setuptools import find_packages, setup


version = __import__("ckeditor").__version__

if sys.argv[-1] == "publish":
    os.system("python setup.py sdist upload")
    os.system("python setup.py bdist_wheel upload")
    sys.exit()

if sys.argv[-1] == "tag":
    os.system(f"git tag -a {version} -m 'version {version}'")
    os.system("git push --tags")
    sys.exit()

long_description = "\n".join(
    [
        open("README.rst").read(),
        open("AUTHORS.rst").read(),
        open("CHANGELOG.rst").read(),
    ]
)


def get_source_files():
    for dirname, _, files in os.walk("ckeditor/static/ckeditor/ckeditor/_source"):
        for filename in files:
            yield os.path.join("/".join(dirname.split("/")[1:]), filename)


setup(
    name="django-ckeditor",
    version=version,
    description="Django admin CKEditor integration.",
    long_description=long_description,
    author="Shaun Sephton & Piotr Malinski",
    author_email="riklaunim@gmail.com",
    url="https://github.com/django-ckeditor/django-ckeditor",
    project_urls={
        "Documentation": "https://django-ckeditor.readthedocs.io/en/latest/",
        "Source": "https://github.com/django-ckeditor/django-ckeditor",
    },
    packages=find_packages(exclude=["*.demo"]),
    zip_safe=False,
    install_requires=[
        "Django>=3.2",
        "django-js-asset>=2.0",
    ],
    python_requires=">=3.8",
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Framework :: Django",
        "Framework :: Django :: 3.2",
        "Framework :: Django :: 4.0",
        "Intended Audience :: Developers",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    ],
)



import os
import os.path
from time import sleep
from unittest import skipUnless

from django.contrib.staticfiles.finders import find
from django.contrib.staticfiles.testing import StaticLiveServerTestCase
from django.test.utils import override_settings
from selenium import webdriver
from selenium.webdriver.common.by import By

from .utils import get_upload_directory, remove_upload_directory, sha1


CHROMIUM = "chromium"
FIREFOX = "firefox"
SELENIUM = os.environ.get("SELENIUM")


@skipUnless(
    SELENIUM,
    "Set the SELENIUM env variable to 'firefox' or 'chromium' to run selenium tests",
)
class TestAdminPanelWidget(StaticLiveServerTestCase):
    fixtures = ["test_admin.json"]

    @classmethod
    def setUpClass(cls):
        if SELENIUM == CHROMIUM:
            options = webdriver.ChromeOptions()
            options.add_argument("--headless")
            options.add_argument("--disable-gpu")
            cls.selenium = webdriver.Chrome(chrome_options=options)
        elif SELENIUM == FIREFOX:
            cls.selenium = webdriver.Firefox()
        super().setUpClass()

    @classmethod
    def tearDownClass(cls):
        cls.selenium.quit()
        super().tearDownClass()

    def setUp(self):
        remove_upload_directory()

    def test_admin_panel_widget(self):
        self._login_to_admin()
        self._go_to_demo_application_in_admin()
        self._assert_editor_loaded()
        self._focus_cursor_in_editor()
        self._enter_test_text()
        self._open_image_upload_widget()
        self._go_to_upload_tab()
        self._switch_to_form_iframe()
        self._upload_image()
        self._assert_image_uploaded()

    def _login_to_admin(self):
        self.selenium.get("{}{}".format(self.live_server_url, "/admin/"))
        username_input = self.selenium.find_element(By.NAME, "username")
        username_input.send_keys("test")
        password_input = self.selenium.find_element(By.NAME, "password")
        password_input.send_keys("test")
        self.selenium.find_element(By.XPATH, '//input[@value="Log in"]').click()

    def _go_to_demo_application_in_admin(self):
        self.selenium.get(
            "{}{}".format(
                self.live_server_url, "/admin/demo_application/examplemodel/add/"
            )
        )

    def _assert_editor_loaded(self):
        sleep(2)
        self.selenium.find_element(By.ID, "cke_id_content")

    def _focus_cursor_in_editor(self):
        self.frame = self.selenium.find_element(By.CLASS_NAME, "cke_wysiwyg_frame")
        self.frame.click()

    def _enter_test_text(self):
        self.frame.send_keys("test")

    def _open_image_upload_widget(self):
        self.selenium.find_element(
            By.CSS_SELECTOR, "span.cke_button_icon.cke_button__image_icon"
        ).click()
        sleep(1)

    def _go_to_upload_tab(self):
        self.selenium.find_element(By.CSS_SELECTOR, "a[title='Upload']").click()
        sleep(1)

    def _switch_to_form_iframe(self):
        iframe = self.selenium.find_element(
            By.CSS_SELECTOR, "iframe.cke_dialog_ui_input_file"
        )
        self.selenium.switch_to.frame(iframe)

    def _upload_image(self):
        input = self.selenium.find_element(By.CSS_SELECTOR, "input[type=file]")
        input.send_keys(self._get_upload_file())
        self.selenium.switch_to.default_content()
        self.selenium.find_element(By.CLASS_NAME, "cke_dialog_ui_fileButton").click()
        sleep(2)

    def _get_upload_file(self):
        return find("ckeditor/ckeditor/skins/moono/images/hidpi/close.png")

    def _assert_image_uploaded(self):
        upload_directory = get_upload_directory()
        expected_image_path = os.path.join(upload_directory, "close.png")
        expected_thumbnail_path = os.path.join(upload_directory, "close_thumb.png")
        self.assertTrue(os.path.isfile(expected_image_path))
        self.assertTrue(os.path.isfile(expected_thumbnail_path))
        self._assert_uploaded_image_did_not_changed(expected_image_path)
        self._assert_thumbnail_is_not_empty(expected_thumbnail_path)
        os.remove(expected_image_path)
        os.remove(expected_thumbnail_path)

    def _assert_uploaded_image_did_not_changed(self, path):
        expected_sha = sha1(self._get_upload_file())
        uploaded_sha = sha1(path)
        self.assertEqual(expected_sha, uploaded_sha)

    def _assert_thumbnail_is_not_empty(self, path):
        size = os.path.getsize(path)
        assert size > 0


@override_settings(CKEDITOR_IMAGE_BACKEND="ckeditor_uploader.backends.DummyBackend")
class TestAdminPanelWidgetForDummyImageBackend(TestAdminPanelWidget):
    def _assert_image_uploaded(self):
        upload_directory = get_upload_directory()
        expected_image_path = os.path.join(upload_directory, "close.png")
        expected_thumbnail_path = os.path.join(upload_directory, "close_thumb.png")
        self.assertTrue(os.path.isfile(expected_image_path), expected_image_path)
        self.assertFalse(
            os.path.exists(expected_thumbnail_path), expected_thumbnail_path
        )
        self._assert_uploaded_image_did_not_changed(expected_image_path)
        os.remove(expected_image_path)



from django.urls import reverse
from django.views import generic

from . import forms


class CkEditorFormView(generic.FormView):
    form_class = forms.CkEditorForm
    template_name = "form.html"

    def get_success_url(self):
        return reverse("ckeditor-form")


class CkEditorMultiWidgetFormView(generic.FormView):
    form_class = forms.CkEditorMultiWidgetForm
    template_name = "form.html"

    def get_success_url(self):
        return reverse("ckeditor-multi-widget-form")


ckeditor_form_view = CkEditorFormView.as_view()
ckeditor_multi_widget_form_view = CkEditorMultiWidgetFormView.as_view()



import json
import os.path

from django.contrib.staticfiles.finders import find
from django.test import TestCase
from django.test.utils import override_settings
from django.urls import reverse

from .utils import get_absolute_media_path, get_media_url, remove_upload_directory, sha1


@override_settings(CKEDITOR_IMAGE_BACKEND="ckeditor_uploader.backends.DummyBackend")
class DummyTestCase(TestCase):
    fixtures = ["test_admin.json"]

    def setUp(self):
        remove_upload_directory()

    def assertJsonResponse(self, response, expected_data):
        self.assertEqual(200, response.status_code)
        data = json.loads(response.content)
        self.assertEqual(data, expected_data)

    def test_upload_file(self):
        self.client.login(username="test", password="test")

        filepath = find("ckeditor/ckeditor/LICENSE.md")

        with open(filepath, "rb") as fp:
            response = self.client.post(reverse("ckeditor_upload"), {"upload": fp})
        self.assertJsonResponse(
            response,
            {
                "fileName": "license.md",
                "uploaded": "1",
                "url": get_media_url("license.md"),
            },
        )
        upload = get_absolute_media_path("license.md")
        self.assertTrue(os.path.isfile(upload), upload)
        self.assertEqual(sha1(filepath), sha1(upload))

    @override_settings(CKEDITOR_ALLOW_NONIMAGE_FILES=False)
    def test_upload_file_disabled(self):
        self.client.login(username="test", password="test")

        filepath = find("ckeditor/ckeditor/LICENSE.md")

        with open(filepath, "rb") as fp:
            response = self.client.post(reverse("ckeditor_upload"), {"upload": fp})
        self.assertEqual(200, response.status_code)
        self.assertContains(response, "Invalid file type.")

    @override_settings(CKEDITOR_ALLOW_NONIMAGE_FILES=False)
    def test_upload_png(self):
        self.client.login(username="test", password="test")

        filepath = find("ckeditor/ckeditor/skins/moono/images/hidpi/close.png")

        with open(filepath, "rb") as fp:
            response = self.client.post(reverse("ckeditor_upload"), {"upload": fp})
        self.assertJsonResponse(
            response,
            {
                "fileName": "close.png",
                "uploaded": "1",
                "url": get_media_url("close.png"),
            },
        )
        upload = get_absolute_media_path("close.png")
        self.assertTrue(os.path.isfile(upload), upload)
        self.assertEqual(sha1(filepath), sha1(upload))

    def test_upload_jpg(self):
        self.client.login(username="test", password="test")

        filepath = find(
            "ckeditor/ckeditor/plugins/codesnippet/lib/highlight/styles/pojoaque.jpg"
        )

        with open(filepath, "rb") as fp:
            response = self.client.post(reverse("ckeditor_upload"), {"upload": fp})
        self.assertJsonResponse(
            response,
            {
                "fileName": "pojoaque.jpg",
                "uploaded": "1",
                "url": get_media_url("pojoaque.jpg"),
            },
        )
        upload = get_absolute_media_path("pojoaque.jpg")
        self.assertTrue(os.path.isfile(upload), upload)
        self.assertEqual(sha1(filepath), sha1(upload))

    def test_upload_gif(self):
        self.client.login(username="test", password="test")

        filepath = find("ckeditor/galleriffic/css/loader.gif")

        with open(filepath, "rb") as fp:
            response = self.client.post(reverse("ckeditor_upload"), {"upload": fp})
        self.assertJsonResponse(
            response,
            {
                "fileName": "loader.gif",
                "uploaded": "1",
                "url": get_media_url("loader.gif"),
            },
        )
        upload = get_absolute_media_path("loader.gif")
        self.assertTrue(os.path.isfile(upload), upload)
        self.assertEqual(sha1(filepath), sha1(upload))
